/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2019  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QCheckBox>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QSpacerItem>

#include "bitfieldwidget.h"

TBitfieldWidget::TBitfieldWidget(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QFrame(const_cast<QWidget*>(parent),flags)
{
	QGridLayout							*bitfield_layout;
	QGridLayout							*layout;
	QHBoxLayout							*value_hlayout;
	QLabel								*value_label;
	QSpacerItem							*value_hspacer;


	layout = new QGridLayout(this);

	value_hlayout = new QHBoxLayout();

	value_label = new QLabel(this);
	value_hlayout->addWidget(value_label);

	d_value_edit = new QLineEdit(this);
	d_value_edit->setAlignment(Qt::AlignCenter);
	value_hlayout->addWidget(d_value_edit);

	value_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	value_hlayout->addItem(value_hspacer);
	layout->addLayout(value_hlayout,0,0,1,1);

	bitfield_layout = new QGridLayout();
	bitfield_layout->setSpacing(0);

	d_b0_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b0_check,0,0,1,1);
	
	d_b1_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b1_check,1,0,1,1);
	
	d_b2_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b2_check,2,0,1,1);

	d_b3_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b3_check,3,0,1,1);
	
	d_b4_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b4_check,4,0,1,1);
	
	d_b5_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b5_check,5,0,1,1);
	
	d_b6_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b6_check,6,0,1,1);

	d_b7_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b7_check,7,0,1,1);

	d_b8_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b8_check,0,1,1,1);
	
	d_b9_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b9_check,1,1,1,1);
	
	d_b10_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b10_check,2,1,1,1);

	d_b11_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b11_check,3,1,1,1);
	
	d_b12_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b12_check,4,1,1,1);
	
	d_b13_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b13_check,5,1,1,1);
	
	d_b14_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b14_check,6,1,1,1);
	
	d_b15_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b15_check,7,1,1,1);

	d_b16_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b16_check,0,2,1,1);
	
	d_b17_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b17_check,1,2,1,1);
	
	d_b18_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b18_check,2,2,1,1);
	
	d_b19_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b19_check,3,2,1,1);
	
	d_b20_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b20_check,4,2,1,1);
	
	d_b21_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b21_check,5,2,1,1);
	
	d_b22_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b22_check,6,2,1,1);

	d_b23_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b23_check,7,2,1,1);

	d_b24_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b24_check,0,3,1,1);

	d_b25_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b25_check,1,3,1,1);

	d_b26_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b26_check,2,3,1,1);

	d_b27_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b27_check,3,3,1,1);

	d_b28_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b28_check,4,3,1,1);

	d_b29_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b29_check,5,3,1,1);

	d_b30_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b30_check,6,3,1,1);

	d_b31_check = new QCheckBox(this);
	bitfield_layout->addWidget(d_b31_check,7,3,1,1);
	
	layout->addLayout(bitfield_layout,1,0,1,1);

	QWidget::setTabOrder(d_value_edit,d_b0_check);
	QWidget::setTabOrder(d_b0_check,d_b1_check);
	QWidget::setTabOrder(d_b1_check,d_b2_check);
	QWidget::setTabOrder(d_b2_check,d_b3_check);
	QWidget::setTabOrder(d_b3_check,d_b4_check);
	QWidget::setTabOrder(d_b4_check,d_b5_check);
	QWidget::setTabOrder(d_b5_check,d_b6_check);
	QWidget::setTabOrder(d_b6_check,d_b7_check);
	QWidget::setTabOrder(d_b7_check,d_b8_check);
	QWidget::setTabOrder(d_b8_check,d_b9_check);
	QWidget::setTabOrder(d_b9_check,d_b10_check);
	QWidget::setTabOrder(d_b10_check,d_b11_check);
	QWidget::setTabOrder(d_b11_check,d_b12_check);
	QWidget::setTabOrder(d_b12_check,d_b13_check);
	QWidget::setTabOrder(d_b13_check,d_b14_check);
	QWidget::setTabOrder(d_b14_check,d_b15_check);
	QWidget::setTabOrder(d_b15_check,d_b16_check);
	QWidget::setTabOrder(d_b16_check,d_b17_check);
	QWidget::setTabOrder(d_b17_check,d_b18_check);
	QWidget::setTabOrder(d_b18_check,d_b19_check);
	QWidget::setTabOrder(d_b19_check,d_b20_check);
	QWidget::setTabOrder(d_b20_check,d_b21_check);
	QWidget::setTabOrder(d_b21_check,d_b22_check);
	QWidget::setTabOrder(d_b22_check,d_b23_check);
	QWidget::setTabOrder(d_b23_check,d_b24_check);
	QWidget::setTabOrder(d_b24_check,d_b25_check);
	QWidget::setTabOrder(d_b25_check,d_b26_check);
	QWidget::setTabOrder(d_b26_check,d_b27_check);
	QWidget::setTabOrder(d_b27_check,d_b28_check);
	QWidget::setTabOrder(d_b28_check,d_b29_check);
	QWidget::setTabOrder(d_b29_check,d_b30_check);
	QWidget::setTabOrder(d_b30_check,d_b31_check);

	this->setWindowTitle(QStringLiteral("Bitfield Widget"));
	
	value_label->setText(QStringLiteral("Value:"));
	d_b0_check->setText(QStringLiteral("B0"));
	d_b8_check->setText(QStringLiteral("B8"));
	d_b16_check->setText(QStringLiteral("B16"));
	d_b24_check->setText(QStringLiteral("B24"));
	d_b25_check->setText(QStringLiteral("B25"));
	d_b9_check->setText(QStringLiteral("B9"));
	d_b17_check->setText(QStringLiteral("B17"));
	d_b1_check->setText(QStringLiteral("B1"));
	d_b26_check->setText(QStringLiteral("B26"));
	d_b10_check->setText(QStringLiteral("B10"));
	d_b18_check->setText(QStringLiteral("B18"));
	d_b2_check->setText(QStringLiteral("B2"));
	d_b27_check->setText(QStringLiteral("B27"));
	d_b11_check->setText(QStringLiteral("B11"));
	d_b19_check->setText(QStringLiteral("B19"));
	d_b3_check->setText(QStringLiteral("B3"));
	d_b28_check->setText(QStringLiteral("B28"));
	d_b12_check->setText(QStringLiteral("B12"));
	d_b20_check->setText(QStringLiteral("B20"));
	d_b4_check->setText(QStringLiteral("B4"));
	d_b29_check->setText(QStringLiteral("B29"));
	d_b13_check->setText(QStringLiteral("B13"));
	d_b21_check->setText(QStringLiteral("B21"));
	d_b5_check->setText(QStringLiteral("B5"));
	d_b30_check->setText(QStringLiteral("B30"));
	d_b14_check->setText(QStringLiteral("B14"));
	d_b22_check->setText(QStringLiteral("B22"));
	d_b6_check->setText(QStringLiteral("B6"));
	d_b31_check->setText(QStringLiteral("B31"));
	d_b15_check->setText(QStringLiteral("B15"));
	d_b23_check->setText(QStringLiteral("B23"));
	d_b7_check->setText(QStringLiteral("B7"));
	
	this->setFrameShape(QFrame::StyledPanel);
	
	// defaults
	d_signals_blocked = false;
	
	connect(d_b0_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b1_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b2_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b3_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b4_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b5_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b6_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b7_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b8_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b9_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b10_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b11_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b12_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b13_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b14_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b15_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b16_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b17_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b18_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b19_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b20_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b21_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b22_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b23_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b24_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b25_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b26_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b27_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b28_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b29_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b30_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
	connect(d_b31_check,SIGNAL(toggled(bool)),this,SLOT(Update_Integer(void)));
}

TBitfieldWidget::~TBitfieldWidget(void)
{
}

int TBitfieldWidget::Integer(void) const
{
	return d_value_edit->text().toInt();
}

void TBitfieldWidget::Reset(void)
{
	d_signals_blocked = true;
	
	d_b0_check->setChecked(false);
	d_b1_check->setChecked(false);
	d_b2_check->setChecked(false);
	d_b3_check->setChecked(false);
	d_b4_check->setChecked(false);
	d_b5_check->setChecked(false);
	d_b6_check->setChecked(false);
	d_b7_check->setChecked(false);
	d_b8_check->setChecked(false);
	d_b9_check->setChecked(false);
	d_b10_check->setChecked(false);
	d_b11_check->setChecked(false);
	d_b12_check->setChecked(false);
	d_b13_check->setChecked(false);
	d_b14_check->setChecked(false);
	d_b15_check->setChecked(false);
	d_b16_check->setChecked(false);
	d_b17_check->setChecked(false);
	d_b18_check->setChecked(false);
	d_b19_check->setChecked(false);
	d_b20_check->setChecked(false);
	d_b21_check->setChecked(false);
	d_b22_check->setChecked(false);
	d_b23_check->setChecked(false);
	d_b24_check->setChecked(false);
	d_b25_check->setChecked(false);
	d_b26_check->setChecked(false);
	d_b27_check->setChecked(false);
	d_b28_check->setChecked(false);
	d_b29_check->setChecked(false);
	d_b30_check->setChecked(false);
	d_b31_check->setChecked(false);
	
	d_signals_blocked = false;
}

void TBitfieldWidget::Set_Integer(
	const int							value)
{
	d_value_edit->setText(QString("%1").arg(value));
	
	d_signals_blocked = true;
	
	(value & 0x00000001) ? d_b0_check->setChecked(true) : d_b0_check->setChecked(false);
	(value & 0x00000002) ? d_b1_check->setChecked(true) : d_b1_check->setChecked(false);
	(value & 0x00000004) ? d_b2_check->setChecked(true) : d_b2_check->setChecked(false);
	(value & 0x00000008) ? d_b3_check->setChecked(true) : d_b3_check->setChecked(false);
	(value & 0x00000010) ? d_b4_check->setChecked(true) : d_b4_check->setChecked(false);
	(value & 0x00000020) ? d_b5_check->setChecked(true) : d_b5_check->setChecked(false);
	(value & 0x00000040) ? d_b6_check->setChecked(true) : d_b6_check->setChecked(false);
	(value & 0x00000080) ? d_b7_check->setChecked(true) : d_b7_check->setChecked(false);
	(value & 0x00000100) ? d_b8_check->setChecked(true) : d_b8_check->setChecked(false);
	(value & 0x00000200) ? d_b9_check->setChecked(true) : d_b9_check->setChecked(false);
	(value & 0x00000400) ? d_b10_check->setChecked(true) : d_b10_check->setChecked(false);
	(value & 0x00000800) ? d_b11_check->setChecked(true) : d_b11_check->setChecked(false);
	(value & 0x00001000) ? d_b12_check->setChecked(true) : d_b12_check->setChecked(false);
	(value & 0x00002000) ? d_b13_check->setChecked(true) : d_b13_check->setChecked(false);
	(value & 0x00004000) ? d_b14_check->setChecked(true) : d_b14_check->setChecked(false);
	(value & 0x00008000) ? d_b15_check->setChecked(true) : d_b15_check->setChecked(false);
	(value & 0x00010000) ? d_b16_check->setChecked(true) : d_b16_check->setChecked(false);
	(value & 0x00020000) ? d_b17_check->setChecked(true) : d_b17_check->setChecked(false);
	(value & 0x00040000) ? d_b18_check->setChecked(true) : d_b18_check->setChecked(false);
	(value & 0x00080000) ? d_b19_check->setChecked(true) : d_b19_check->setChecked(false);
	(value & 0x00100000) ? d_b20_check->setChecked(true) : d_b20_check->setChecked(false);
	(value & 0x00200000) ? d_b21_check->setChecked(true) : d_b21_check->setChecked(false);
	(value & 0x00400000) ? d_b22_check->setChecked(true) : d_b22_check->setChecked(false);
	(value & 0x00800000) ? d_b23_check->setChecked(true) : d_b23_check->setChecked(false);
	(value & 0x01000000) ? d_b24_check->setChecked(true) : d_b24_check->setChecked(false);
	(value & 0x02000000) ? d_b25_check->setChecked(true) : d_b25_check->setChecked(false);
	(value & 0x04000000) ? d_b26_check->setChecked(true) : d_b26_check->setChecked(false);
	(value & 0x08000000) ? d_b27_check->setChecked(true) : d_b27_check->setChecked(false);
	(value & 0x10000000) ? d_b28_check->setChecked(true) : d_b28_check->setChecked(false);
	(value & 0x20000000) ? d_b29_check->setChecked(true) : d_b29_check->setChecked(false);
	(value & 0x40000000) ? d_b30_check->setChecked(true) : d_b30_check->setChecked(false);
	(value & 0x80000000) ? d_b31_check->setChecked(true) : d_b31_check->setChecked(false);
	
	d_signals_blocked = false;
}

void TBitfieldWidget::Update_Integer(void)
{
	int									value(0);
	
	if(d_b0_check->isChecked()) value |= 0x00000001;
	if(d_b1_check->isChecked()) value |= 0x00000002;
	if(d_b2_check->isChecked()) value |= 0x00000004;
	if(d_b3_check->isChecked()) value |= 0x00000008;
	if(d_b4_check->isChecked()) value |= 0x00000010;
	if(d_b5_check->isChecked()) value |= 0x00000020;
	if(d_b6_check->isChecked()) value |= 0x00000040;
	if(d_b7_check->isChecked()) value |= 0x00000080;
	if(d_b8_check->isChecked()) value |= 0x00000100;
	if(d_b9_check->isChecked()) value |= 0x00000200;
	if(d_b10_check->isChecked()) value |= 0x00000400;
	if(d_b11_check->isChecked()) value |= 0x00000800;
	if(d_b12_check->isChecked()) value |= 0x00001000;
	if(d_b13_check->isChecked()) value |= 0x00002000;
	if(d_b14_check->isChecked()) value |= 0x00004000;
	if(d_b15_check->isChecked()) value |= 0x00008000;
	if(d_b16_check->isChecked()) value |= 0x00010000;
	if(d_b17_check->isChecked()) value |= 0x00020000;
	if(d_b18_check->isChecked()) value |= 0x00040000;
	if(d_b19_check->isChecked()) value |= 0x00080000;
	if(d_b20_check->isChecked()) value |= 0x00100000;
	if(d_b21_check->isChecked()) value |= 0x00200000;
	if(d_b22_check->isChecked()) value |= 0x00400000;
	if(d_b23_check->isChecked()) value |= 0x00800000;
	if(d_b24_check->isChecked()) value |= 0x01000000;
	if(d_b25_check->isChecked()) value |= 0x02000000;
	if(d_b26_check->isChecked()) value |= 0x04000000;
	if(d_b27_check->isChecked()) value |= 0x08000000;
	if(d_b28_check->isChecked()) value |= 0x10000000;
	if(d_b29_check->isChecked()) value |= 0x20000000;
	if(d_b30_check->isChecked()) value |= 0x40000000;
	if(d_b31_check->isChecked()) value |= 0x80000000;
	
	d_value_edit->setText(QString("%1").arg(value));

	if(!d_signals_blocked)
	{
		emit Value_Changed(0);
	}
}


