/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2019  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef CONTROLLERPARAMETERSHEADERFILE
#define CONTROLLERPARAMETERSHEADERFILE

#include <QMainWindow>

#include "rules.h"

#include "parameters.h"

class QAction;
class QComboBox;
class QLabel;
class QLineEdit;
class QListWidget;
class QPlainTextEdit;
class QStackedWidget;
class QStatusBar;
class QFrame;
class QToolButton;
class QSettings;
class QCloseEvent;
class QDragEnterEvent;
class QDropEvent;
class QSplitter;
class TBitfieldWidget;
class TValuesWidget;
class TRuleEditorDialog;
class TCreateGroupDialog;
class TExportOptionsDialog;
class TMessageBox;

class TControllerParameters : public QMainWindow
{
	Q_OBJECT
	
public:
	
	// CREATORS
	TControllerParameters(const QWidget *parent=0, Qt::WindowFlags flags=0);
	virtual ~TControllerParameters(void);
	
	// ACCESSORS
	
	// MANIPULATORS
	
signals:
	
private slots:
	void File_New(void);
	void File_Load_Parameters(void);
	void File_Export(void);

	void Parameter_Selection_Changed(void);
	void Search_Changed(const QString&);
	void Value_Changed(const int);
	
	void Add_Group(void);
	void Remove_Group(void);
	void Edit_Rules(void);
	void Execute_Rules(void);

protected:
	void closeEvent(QCloseEvent *event);
	void dragEnterEvent(QDragEnterEvent *drag_enter_event);
	void dropEvent(QDropEvent *dropEvent);

private:
	QAction								*d_file_new_action;
	QAction								*d_file_open_action;
	QAction								*d_file_export_action;
	QAction								*d_file_execute_rules_action;
	QAction								*d_file_quit_action;
	QAction								*d_edit_rules_action;
	QComboBox							*d_rules_combo;
	QLabel								*d_parameter_help_label;
	QLabel								*d_parameter_name_label;
	QLineEdit							*d_search_edit;
	QListWidget							*d_parameter_list;
	QPlainTextEdit						*d_rules_result_edit;
	QPlainTextEdit						*d_variable_list_edit;
	QStackedWidget						*d_parameter_value_stack;
	QStatusBar							*d_status_bar;
	QToolButton							*d_add_group_button;
	QToolButton							*d_remove_group_button;
	QToolButton							*d_edit_rules_button;
	QToolButton							*d_execute_rules_button;
	QFrame								*d_empty_widget;
	QSplitter							*d_splitter_widget;
	TBitfieldWidget						*d_bitfield_widget;
	TValuesWidget						*d_values_widget;
	TRuleEditorDialog					*d_rule_editor_dialog;
	TCreateGroupDialog					*d_create_group_dialog;
	TExportOptionsDialog				*d_export_options_dialog;
	TMessageBox							*d_msg_box;
	
	QSettings							*d_settings;
	TRules								d_rules;
	QString								d_settings_path;
	TParameters							d_parameters;
	QString								d_active_parameter_name;
	
	void Create_Actions(void);
	
	void Update_Variable_List(void);
	
	void Add_Rules_Text(const QString &display_text,const bool bold=false);
	void Add_Variables_Text(const QString &display_text);

		
	// NOT IMPLEMENTED
	TControllerParameters(const TControllerParameters&);
	TControllerParameters& operator=(const TControllerParameters&);
};

#endif
