/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2019  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QComboBox>
#include <QFrame>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QSizePolicy>
#include <QSpacerItem>
#include <QStringList>

#include "copyrulesdialog.h"

TCopyRulesDialog::TCopyRulesDialog(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QFrame								*separator_hline;
	QGridLayout							*dialog_layout;
	QHBoxLayout							*button_hlayout;
	QLabel								*target_group_label;
	QSizePolicy							size_policy(QSizePolicy::Maximum,QSizePolicy::Preferred);
	QSpacerItem							*button_hspacer;
	QSpacerItem							*dialog_vspacer;

	size_policy.setHorizontalStretch(0);
	size_policy.setVerticalStretch(0);

	this->resize(408,118);

	dialog_layout = new QGridLayout(this);

	target_group_label = new QLabel(this);
	target_group_label->setSizePolicy(size_policy);
	dialog_layout->addWidget(target_group_label,0,0,1,1);

	d_group_combo = new QComboBox(this);
	dialog_layout->addWidget(d_group_combo,0,1,1,1);

	dialog_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	dialog_layout->addItem(dialog_vspacer,1,0,1,2);

	separator_hline = new QFrame(this);
	separator_hline->setFrameShape(QFrame::HLine);
	separator_hline->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_hline,2,0,1,2);

	button_hlayout = new QHBoxLayout();

	button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_hlayout->addItem(button_hspacer);

	d_cancel_button = new QPushButton(this);
	d_cancel_button->setAutoDefault(false);
	button_hlayout->addWidget(d_cancel_button);

	d_copy_button = new QPushButton(this);
	d_copy_button->setAutoDefault(true);
	button_hlayout->addWidget(d_copy_button);
	dialog_layout->addLayout(button_hlayout,3,0,1,2);

	this->setWindowTitle(QStringLiteral("Copy Rules"));
	
	target_group_label->setText(QStringLiteral("Target Group:"));
	d_cancel_button->setText(QStringLiteral("Cancel"));
	d_copy_button->setText(QStringLiteral("Copy"));

	connect(d_cancel_button,SIGNAL(clicked(void)),this,SLOT(reject(void)));
	connect(d_copy_button,SIGNAL(clicked(void)),this,SLOT(accept(void)));
}

TCopyRulesDialog::~TCopyRulesDialog(void)
{
}

QString TCopyRulesDialog::Target_Group(void) const
{
	return d_group_combo->currentText();
}

void TCopyRulesDialog::Reset(
	const QStringList					&available_groups)
{
	d_group_combo->clear();
	d_group_combo->addItems(available_groups);
}
