/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2019  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QFrame>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QSpacerItem>

#include "creategroupdialog.h"

TCreateGroupDialog::TCreateGroupDialog(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QFrame								*separator_hline;
	QGridLayout							*dialog_layout;
	QHBoxLayout							*button_hlayout;
	QLabel								*group_name_label;
	QSpacerItem							*button_hspacer;
	QSpacerItem							*dialog_vspacer;


	this->resize(408,118);

	dialog_layout = new QGridLayout(this);

	group_name_label = new QLabel(this);
	dialog_layout->addWidget(group_name_label,0,0,1,1);

	separator_hline = new QFrame(this);
	separator_hline->setFrameShape(QFrame::HLine);
	separator_hline->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_hline,2,0,1,2);

	d_group_name_edit = new QLineEdit(this);
	dialog_layout->addWidget(d_group_name_edit,0,1,1,1);

	button_hlayout = new QHBoxLayout();

	button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_hlayout->addItem(button_hspacer);

	d_cancel_button = new QPushButton(this);
	d_cancel_button->setAutoDefault(false);
	button_hlayout->addWidget(d_cancel_button);

	d_create_button = new QPushButton(this);
	d_create_button->setAutoDefault(true);
	button_hlayout->addWidget(d_create_button);
	dialog_layout->addLayout(button_hlayout,3,0,1,2);

	dialog_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	dialog_layout->addItem(dialog_vspacer,1,0,1,2);

	this->setWindowTitle(QStringLiteral("Create Group"));
	
	group_name_label->setText(QStringLiteral("Group Name:"));
	d_cancel_button->setText(QStringLiteral("Cancel"));
	d_create_button->setText(QStringLiteral("Create"));
	
	connect(d_group_name_edit,SIGNAL(textEdited(const QString&)),this,SLOT(Validate(const QString&)));
	connect(d_cancel_button,SIGNAL(clicked(void)),this,SLOT(reject(void)));
	connect(d_create_button,SIGNAL(clicked(void)),this,SLOT(accept(void)));

}

TCreateGroupDialog::~TCreateGroupDialog(void)
{
}

QString TCreateGroupDialog::Group_Name(void) const
{
	return d_group_name_edit->text();
}

void TCreateGroupDialog::Reset(
	const QStringList					&existing_groups)
{
	d_existing_groups = existing_groups;
	
	d_group_name_edit->setText(QString());
	d_create_button->setEnabled(false);
}

void TCreateGroupDialog::Validate(
	const QString						&text)
{
	d_create_button->setEnabled(!d_existing_groups.contains(text,Qt::CaseInsensitive));
}

