/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2019  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QCheckBox>
#include <QFrame>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QRadioButton>
#include <QSizePolicy>
#include <QSpacerItem>
#include <QVBoxLayout>
#include <QWidget>

#include "exportoptionsdialog.h"

TExportOptionsDialog::TExportOptionsDialog(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QFrame								*separator_hline;
	QGridLayout							*check_widget_layout;
	QGridLayout							*dialog_layout;
	QHBoxLayout							*button_hlayout;
	QLabel								*options_label;
	QSizePolicy							size_policy(QSizePolicy::Maximum,QSizePolicy::Preferred);
	QSpacerItem							*button_hspacer;
	QSpacerItem							*dialog_vspacer;
	QVBoxLayout							*radio_widget_layout;
	QWidget								*check_widget_container;
	QWidget								*radio_widget_container;

	size_policy.setHorizontalStretch(0);
	size_policy.setVerticalStretch(0);

	this->resize(595,235);

	dialog_layout = new QGridLayout(this);

	options_label = new QLabel(this);
	options_label->setSizePolicy(size_policy);
	options_label->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
	dialog_layout->addWidget(options_label,0,0,2,1);

	radio_widget_container = new QWidget(this);

	radio_widget_layout = new QVBoxLayout(radio_widget_container);
	radio_widget_layout->setContentsMargins(0,0,0,0);

	d_export_changed_radio = new QRadioButton(radio_widget_container);
	d_export_changed_radio->setChecked(true);
	radio_widget_layout->addWidget(d_export_changed_radio);

	d_export_all_radio = new QRadioButton(radio_widget_container);
	radio_widget_layout->addWidget(d_export_all_radio);
	dialog_layout->addWidget(radio_widget_container,0,1,1,1);

	check_widget_container = new QWidget(this);

	check_widget_layout = new QGridLayout(check_widget_container);
	check_widget_layout->setContentsMargins(0,0,0,0);

	d_include_original_check = new QCheckBox(check_widget_container);
	d_include_original_check->setChecked(true);
	check_widget_layout->addWidget(d_include_original_check,0,0,1,1);

	d_match_input_check = new QCheckBox(check_widget_container);
	d_match_input_check->setChecked(true);
	check_widget_layout->addWidget(d_match_input_check,1,0,1,1);
	dialog_layout->addWidget(check_widget_container,1,1,1,1);

	dialog_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	dialog_layout->addItem(dialog_vspacer,2,0,1,2);

	separator_hline = new QFrame(this);
	separator_hline->setFrameShape(QFrame::HLine);
	separator_hline->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_hline,3,0,1,2);

	button_hlayout = new QHBoxLayout();

	button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_hlayout->addItem(button_hspacer);

	d_cancel_button = new QPushButton(this);
	d_cancel_button->setAutoDefault(false);
	button_hlayout->addWidget(d_cancel_button);

	d_continue_button = new QPushButton(this);
	d_continue_button->setAutoDefault(true);
	button_hlayout->addWidget(d_continue_button);
	dialog_layout->addLayout(button_hlayout,4,0,1,2);

	this->setWindowTitle(QStringLiteral("Export Options"));
	
	options_label->setText(QStringLiteral("Options:"));
	d_export_changed_radio->setText(QStringLiteral("Export changed items only"));
	d_export_all_radio->setText(QStringLiteral("Export all data"));
	d_include_original_check->setText(QStringLiteral("Include original values for changed items"));
	d_match_input_check->setText(QStringLiteral("Output format matches input format"));
	d_cancel_button->setText(QStringLiteral("Cancel"));
	d_continue_button->setText(QStringLiteral("Continue"));
	
	connect(d_cancel_button,SIGNAL(clicked(void)),this,SLOT(reject(void)));
	connect(d_continue_button,SIGNAL(clicked(void)),this,SLOT(accept(void)));

}

TExportOptionsDialog::~TExportOptionsDialog(void)
{
}

Utilities::TExportOptions TExportOptionsDialog::Export_Options(void) const
{
	Utilities::TExportOptions			export_options;
	
	export_options.export_changed_only = d_export_changed_radio->isChecked();
	export_options.export_original_values = d_include_original_check->isChecked();
	export_options.export_format_match_input = d_match_input_check->isChecked();
	
	return export_options;
}

void TExportOptionsDialog::Reset(
	const Utilities::TExportOptions		&export_options)
{
	if(export_options.export_changed_only)
	{
		d_export_changed_radio->setChecked(true);
	}
	else
	{
		d_export_all_radio->setChecked(true);
	}
	
	d_include_original_check->setChecked(export_options.export_original_values);
	d_match_input_check->setChecked(export_options.export_format_match_input);
}
