/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2019  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef PARAMETERSHEADERFILE
#define PARAMETERSHEADERFILE

#include <QString>
#include <vector>

#include "../../core/expression.h"

#include "utilities.h"

class TXmlFile;

class TParameters
{
public:
	
enum TParameterType
{
	TYPE_UNKNOWN = 0,
	TYPE_INTEGER,
	TYPE_FLOAT,
	TYPE_TEXT,
	TYPE_INTEGER_ARRAY,
	TYPE_FLOAT_ARRAY,
	TYPE_TEXT_ARRAY,
	TYPE_BIT_PATTERN
};
	
enum TParameterFileFormat
{
	FORMAT_UNKNOWN=0,
	FORMAT_CONSTANTS,
	FORMAT_XML
};
	
struct TParameter
{
	QString								name;
	QString								extended_name;
	QString								description;
	TParameterType						type;
	int									index;
	bool								changed;
	
	std::vector<int>					values_int;
	std::vector<double>					values_float;
	std::vector<QString>				values_text;
	
	std::vector<int>					values_int_orig;
	std::vector<double>					values_float_orig;
	std::vector<QString>				values_text_orig;
};
	
// CREATORS
	TParameters(void);
	~TParameters(void);
	
// ACCESSORS
	QString Last_Error(void) const {return d_last_error;}
	
	QStringList Get_Parameter_List(const QString &filter = QString()) const;
	TParameter Get_Parameter(const QString &name,bool * const state = 0) const;	// state false if item not found
	std::vector<TExpression::TVariable> Get_Variables(int * const max_width = 0) const;
	
	bool Write_Parameter_Data(const QString &file_name,Utilities::TExportOptions &export_options) const;

// MANIPULATORS
	void Reset(void);
	
	bool Load_Parameter_File(const QString &file_name);
	void Update_Parameter(const TParameter &parameter);
	
private:
	std::vector<TParameter>				d_parameters;
	TParameterFileFormat				d_parameters_format;
	
	mutable QString						d_last_error;
	
	QString Get_Constants_Description(const int index) const;
	QString Format_Value(const QString &in) const;
	TParameter Convert_From_Text(const TParameter &parameter) const;	// turn Text entry into Integer or Float if possible

	
	bool Load_Constants_File(const QByteArray &file_data);
	bool Load_XML_File(const QString &file_name);
	
	bool Add_Parameter(const TParameter &parameter);

	
// NOT IMPLEMENTED
	TParameters(const TParameters&);
	TParameters& operator=(const TParameters&);
};

#endif
