/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2019  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QFrame>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QRadioButton>
#include <QSpacerItem>
#include <QWidget>
#include <assert.h>

#include "ruleitemdialog.h"

TRuleItemDialog::TRuleItemDialog(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QFrame								*separator_lower_line;
	QFrame								*seperator_upper_line;
	QGridLayout							*dialog_layout;
	QGridLayout							*operator_layout;
	QHBoxLayout							*button_hlayout;
	QLabel								*help_label;
	QLabel								*left_expression_label;
	QLabel								*operator_label;
	QLabel								*right_expression_label;
	QLabel								*rule_name_label;
	QSpacerItem							*button_hspacer;
	QWidget								*operator_widget;


	this->resize(665,301);

	dialog_layout = new QGridLayout(this);

	operator_widget = new QWidget(this);

	operator_layout = new QGridLayout(operator_widget);

	d_less_radio = new QRadioButton(operator_widget);
	operator_layout->addWidget(d_less_radio,0,0,1,1);
	
	d_not_greater_radio = new QRadioButton(operator_widget);
	operator_layout->addWidget(d_not_greater_radio,0,1,1,1);

	d_equal_radio = new QRadioButton(operator_widget);
	operator_layout->addWidget(d_equal_radio,1,0,1,1);

	d_not_equal_radio = new QRadioButton(operator_widget);
	operator_layout->addWidget(d_not_equal_radio,1,1,1,1);

	d_greater_radio = new QRadioButton(operator_widget);
	operator_layout->addWidget(d_greater_radio,2,0,1,1);
	
	d_not_less_radio = new QRadioButton(operator_widget);
	operator_layout->addWidget(d_not_less_radio,2,1,1,1);
	
	dialog_layout->addWidget(operator_widget,3,2,1,1);

	seperator_upper_line = new QFrame(this);
	seperator_upper_line->setFrameShape(QFrame::HLine);
	seperator_upper_line->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(seperator_upper_line,1,0,1,3);

	left_expression_label = new QLabel(this);
	dialog_layout->addWidget(left_expression_label,2,0,1,1);

	d_left_expression_edit = new QLineEdit(this);
	dialog_layout->addWidget(d_left_expression_edit,2,2,1,1);

	operator_label = new QLabel(this);
	operator_label->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
	dialog_layout->addWidget(operator_label,3,0,1,1);

	right_expression_label = new QLabel(this);
	dialog_layout->addWidget(right_expression_label,4,0,1,1);

	d_right_expression_edit = new QLineEdit(this);
	dialog_layout->addWidget(d_right_expression_edit,4,2,1,1);

	separator_lower_line = new QFrame(this);
	separator_lower_line->setFrameShape(QFrame::HLine);
	separator_lower_line->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_lower_line,6,0,1,3);

	button_hlayout = new QHBoxLayout();

	button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_hlayout->addItem(button_hspacer);

	d_cancel_button = new QPushButton(this);
	d_cancel_button->setAutoDefault(false);
	button_hlayout->addWidget(d_cancel_button);

	d_apply_button = new QPushButton(this);
	d_apply_button->setAutoDefault(false);
	button_hlayout->addWidget(d_apply_button);
	dialog_layout->addLayout(button_hlayout,7,0,1,3);

	d_rule_name_edit = new QLineEdit(this);
	dialog_layout->addWidget(d_rule_name_edit,0,2,1,1);

	rule_name_label = new QLabel(this);
	dialog_layout->addWidget(rule_name_label,0,0,1,1);

	help_label = new QLabel(this);
	dialog_layout->addWidget(help_label,5,2,1,1);

	QWidget::setTabOrder(d_rule_name_edit,d_left_expression_edit);
	QWidget::setTabOrder(d_left_expression_edit,d_less_radio);
	QWidget::setTabOrder(d_less_radio,d_equal_radio);
	QWidget::setTabOrder(d_equal_radio,d_greater_radio);
	QWidget::setTabOrder(d_greater_radio,d_not_greater_radio);
	QWidget::setTabOrder(d_not_greater_radio,d_not_equal_radio);
	QWidget::setTabOrder(d_not_equal_radio,d_not_less_radio);
	QWidget::setTabOrder(d_not_less_radio,d_right_expression_edit);
	QWidget::setTabOrder(d_right_expression_edit,d_cancel_button);
	QWidget::setTabOrder(d_cancel_button,d_apply_button);

	this->setWindowTitle(QStringLiteral("Rule Item Editor"));
	
	d_less_radio->setText(QStringLiteral("Left Exp. Less Than Right Exp."));
	d_not_less_radio->setText(QStringLiteral("Left Exp. Not Less Than Right Exp."));
	d_equal_radio->setText(QStringLiteral("Left Exp. Equals Right Exp."));
	d_not_equal_radio->setText(QStringLiteral("Left Exp. Not Equal To Right Exp."));
	d_greater_radio->setText(QStringLiteral("Left Exp. Greater Than Right Exp."));
	d_not_greater_radio->setText(QStringLiteral("Left Exp. Not Greater Than Right Exp."));
	left_expression_label->setText(QStringLiteral("Left Expression:"));
	operator_label->setText(QStringLiteral("Operator:"));
	right_expression_label->setText(QStringLiteral("Right Expression:"));
	d_cancel_button->setText(QStringLiteral("Cancel"));
	d_apply_button->setText(QStringLiteral("Apply"));
	rule_name_label->setText(QStringLiteral("Rule Name:"));
	help_label->setText(QStringLiteral("<html><head/><body><p>Note:  Result of <span style=\" font-weight:600; font-style:italic;\">[Left Expression] &lt;operator&gt; [Right Expression]</span> must be true.</p></body></html>"));
	
	connect(d_apply_button,SIGNAL(clicked(void)),this,SLOT(accept(void)));
	connect(d_cancel_button,SIGNAL(clicked(void)),this,SLOT(reject(void)));
	
	connect(d_less_radio,SIGNAL(toggled(bool)),this,SLOT(Options_Changed(void)));
	connect(d_equal_radio,SIGNAL(toggled(bool)),this,SLOT(Options_Changed(void)));
	connect(d_greater_radio,SIGNAL(toggled(bool)),this,SLOT(Options_Changed(void)));
	connect(d_not_less_radio,SIGNAL(toggled(bool)),this,SLOT(Options_Changed(void)));
	connect(d_not_equal_radio,SIGNAL(toggled(bool)),this,SLOT(Options_Changed(void)));
	connect(d_not_greater_radio,SIGNAL(toggled(bool)),this,SLOT(Options_Changed(void)));
	
	connect(d_rule_name_edit,SIGNAL(textEdited(const QString&)),SLOT(Options_Changed(void)));
	connect(d_left_expression_edit,SIGNAL(textEdited(const QString&)),SLOT(Options_Changed(void)));
	connect(d_right_expression_edit,SIGNAL(textEdited(const QString&)),SLOT(Options_Changed(void)));
}

TRuleItemDialog::~TRuleItemDialog(void)
{
}

TRules::TRuleItem TRuleItemDialog::Rule_Item(void) const
{
	TRules::TRuleItem					rule_item;
	
	rule_item.name = d_rule_name_edit->text().simplified();
	rule_item.left_expression = d_left_expression_edit->text().simplified();
	rule_item.right_expression = d_right_expression_edit->text().simplified();
	
	if(d_less_radio->isChecked())
	{
		rule_item.rule_operator = TRules::OPERATOR_LESS_THAN;
	}
	else if(d_equal_radio->isChecked())
	{
		rule_item.rule_operator = TRules::OPEATOR_EQUAL;
	}
	else if(d_greater_radio->isChecked())
	{
		rule_item.rule_operator = TRules::OPERATOR_GREATER_THAN;
	}
	else if(d_not_less_radio->isChecked())
	{
		rule_item.rule_operator = TRules::OPERATOR_NOT_LESS_THAN;
	}
	else if(d_not_equal_radio->isChecked())
	{
		rule_item.rule_operator = TRules::OPERATOR_NOT_EQUAL;
	}
	else
	{
		rule_item.rule_operator = TRules::OPERATOR_NOT_GREATER_THAN;
	}
	
	
	return rule_item;
}

void TRuleItemDialog::Reset(
	const TRuleItemDialog::TMode		mode,
	const TRules::TRuleItem				&rule_item,
	const QStringList					&rule_item_names)
{
	d_rule_name_edit->setEnabled(true);

	d_mode = mode;
	d_rule_item_names = rule_item_names;

	switch(d_mode)
	{
		case TRuleItemDialog::MODE_EDIT:
			d_active_rule_name = rule_item.name;
			break;
			
		case TRuleItemDialog::MODE_ADD:
			d_active_rule_name = QString();
			return;
	}
	
	d_rule_name_edit->setText(rule_item.name);
	d_left_expression_edit->setText(rule_item.left_expression);
	d_right_expression_edit->setText(rule_item.right_expression);
	
	switch(rule_item.rule_operator)
	{
		case TRules::OPERATOR_LESS_THAN:
			d_less_radio->setChecked(true);
			break;
		case TRules::OPEATOR_EQUAL:
			d_equal_radio->setChecked(true);
			break;
		case TRules::OPERATOR_GREATER_THAN:
			d_greater_radio->setChecked(true);
			break;
		case TRules::OPERATOR_NOT_LESS_THAN:
			d_not_less_radio->setChecked(true);
			break;
		case TRules::OPERATOR_NOT_EQUAL:
			d_not_equal_radio->setChecked(true);
			break;
		case TRules::OPERATOR_NOT_GREATER_THAN:
			d_not_greater_radio->setChecked(true);
			break;
	}
	
	d_apply_button->setEnabled(false);
}

void TRuleItemDialog::Options_Changed(void)
{
	d_apply_button->setEnabled(this->Validate());
}

bool TRuleItemDialog::Validate(void) const
{
	QString								text;
	
	text = d_rule_name_edit->text().simplified();
	
	if(!text.length())
	{
		return false;
	}

	// rule names longer than 64 char's seems a bit excessive
	if(text.length() > 64)
	{
		return false;
	}

	switch(d_mode)
	{
		case TRuleItemDialog::MODE_EDIT:
			if(text.compare(d_active_rule_name) != 0)
			{
				if(d_rule_item_names.contains(text,Qt::CaseInsensitive))
				{
					return false;
				}
			}

			break;
			
		case TRuleItemDialog::MODE_ADD:
			if(d_rule_item_names.contains(text,Qt::CaseInsensitive))
			{
				return false;
			}
			break;
	}
	
	if(d_less_radio->isChecked() == false &&
	   d_equal_radio->isChecked() == false &&
	   d_greater_radio->isChecked() == false &&
	   d_not_less_radio->isChecked() == false &&
	   d_not_equal_radio->isChecked() == false &&
	   d_not_greater_radio->isChecked() == false)
	{
		return false;
	}
	
	text = d_left_expression_edit->text().simplified();
	
	if(text.length() == 0)
	{
		return false;
	}
	
	text = d_right_expression_edit->text().simplified();
	
	if(text.length() == 0)
	{
		return false;
	}

	return true;
}

