/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2019  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef RULESHEADERFILE
#define RULESHEADERFILE

#include <QString>
#include <QStringList>
#include <vector>

class TXmlFile;

class TRules
{
public:
	
enum TRuleOperator
{
	OPERATOR_LESS_THAN=0,
	OPEATOR_EQUAL,
	OPERATOR_GREATER_THAN,
	OPERATOR_NOT_LESS_THAN,
	OPERATOR_NOT_EQUAL,
	OPERATOR_NOT_GREATER_THAN
};
	
struct TRuleItem
{
	QString								name;
	QString								left_expression;
	QString								right_expression;
	TRuleOperator						rule_operator;
};
	
struct TRule
{
	QString								name;
	std::vector<TRuleItem>				rule_items;
};

// CREATORS
	TRules(void);
	~TRules(void);
	
// ACCESSORS
	QString Last_Error(void) const {return d_last_error;}
	bool Save_Rules(void) const;
	
	QStringList Rule_Groups(void) const;
	QStringList Rule_List(const QString &rule_group) const;
	
	TRuleItem Rule(const QString &rule_group,const QString &rule_name,bool *valid = 0) const;
	std::vector<TRuleItem> Rules(const QString &rule_group) const;

// MANIPULATORS
	void Set_Rule_Path(const QString &path);
	bool Load_Rules(void);
	
	bool Create_Group(const QString &group_name);
	void Remove_Group(const QString &group_name);
	
	bool Add_Rule(const QString &rule_group,const TRuleItem &rule_item);
	void Delete_Rule(const QString &rule_group,const QString &rule_name);
	bool Copy_Rule(const QString &source_rule_group,const QString &destination_rule_group,const QString &rule_name,const bool overwrite_existing);
	
	void Data_Changed(void) {d_data_changed = true;}
	
private:
	
	QString								d_rule_path;
	std::vector<TRule>					d_rules;
	
	mutable bool						d_data_changed;
	mutable QString						d_last_error;
	
	bool Write(void) const;
	
	void Load_Version1(TXmlFile * const xml_file);
	void Sort_Rules(void);
	
// NOT IMPLEMENTED
	TRules(const TRules&);
	TRules& operator=(const TRules&);
};

#endif