/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2019  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef EXPRESSIONHEADERFILE
#define EXPRESSIONHEADERFILE

#include <vector>
#include <stack>
#include <QString>

class TExpression
{
public:
struct TVariable
{
	QString								name;
	double								value;
};
	
// CREATORS
	TExpression(void);
	~TExpression(void);
	
// ACCESSORS
	int Variable_Count(void) const {return static_cast<int>(d_variables.size());}
	QString Variable_Name(const int index) const;
	double Variable_Value(const int index) const;
	double Variable_Value(const QString &name,bool *exists) const;
	TVariable Variable(const int index) const;
	
	// Function SUMSQ(,,,) or SQRT() calculated if found
	// These are the only functions other than operators +-/*
	double Evaluate(const QString &expression,bool *valid) const;
	
// MANIPULATORS
	void Clear_Variables(void) {d_variables.clear();}
	bool Add_Variable(const TVariable &variable);
	bool Add_Variable(const QString &name,const double &value);
	
private:
	std::vector<TVariable>				d_variables;
	
	double Process_Expression(const QString &input,bool *valid) const;
	double Apply_Expression(std::stack<double> *number_stack,std::stack<int> *oper_stack) const;
	int Operator_Precidence(const int oper) const;
	double Process_Function(const QString &string,int *function_length,bool *valid) const;
	int Find_Closing_Brace(const QString &input,const int open_brace_index) const;


// NOT IMPLEMENTED
	TExpression(const TExpression&);
	TExpression& operator=(const TExpression&);
};

#endif

